/*****************************************************************************************
 *
 *  Copyright 2006+ by Cieszak
 *  All Rights Reserved.
 *
 *  Version    Date          Author             Log
 *  --------------------------------------------------------------------------------------
 *  1.0        2006.07.16    Cieszak              Inital Release
 *
 ****************************************************************************************/

#include "main.h"

#define DEFAULT_STATE (21845+21845*16384)
#define filename "pkp.ini"

/**************************************************************************************/

static void init()
{
   if(TFS_SIZE(tfsstat(filename))!=sizeof(app))
   {
      //default settings (Olsztyn<->Iawa)
      app.state=DEFAULT_STATE;
      app.days=1;
      app.passages=1;
      app.daysinmonth=31;
      app.monthcost=143.62;
      app.oncecost=6.615;
      app.buffer[0]=0;
      //save to file
      tfsadd(filename, TFS_A_HIDDEN, &app, sizeof(app));
   }
   else
   {
      void* buf=tfsload(filename);
      memcpy(&app, buf , sizeof(app));
      free(buf);
   }
}

static void save()
{
   tfsadd(filename, TFS_A_HIDDEN, &app, sizeof(app));
}

/**************************************************************************************/

/* UI messages dispatcher */
extern rexx_uint id_days;
extern rexx_uint id_passages;
extern rexx_uint id_daysinmonth;
extern rexx_uint id_monthcost;
extern rexx_uint id_oncecost;
extern rexx_uint id_calculate;

extern struct dialog dlg_days;
extern struct dialog dlg_passages;
extern struct dialog dlg_daysinmonth;
extern struct dialog dlg_monthcost;
extern struct dialog dlg_oncecost;

extern const nokstr_t str_msg_wrong_number;
extern const nokstr_t str_msg_monthly;
extern const nokstr_t str_msg_daily;
extern const nokstr_t str_msg_cannot_ret;


rexx("id_days=add_5e0_id(dispatcher_days)");
rexx("id_passages=add_5e0_id(dispatcher_passages)");
rexx("id_daysinmonth=add_5e0_id(dispatcher_daysinmonth)");
rexx("id_monthcost=add_5e0_id(dispatcher_monthcost)");
rexx("id_oncecost=add_5e0_id(dispatcher_oncecost)");
rexx("id_calculate=add_5e0_id(dispatcher_calculate)");


/*** days ***/
ULONG dispatcher_days(ULONG msg)
{
   switch(msg)
   {
      case MSG_DISPLAY_MENU_VALUE:
      {
         if(app.state!=DEFAULT_STATE)
         {
            init();
         }
         ftoa(app.buffer, app.days);
         own_print_menu_value((const nokstr_t*)app.buffer);
         break;
      }
      case MSG_D_INIT:
      {
         if(app.state!=DEFAULT_STATE)
         {
            init();
         }
         app.buffer[0]=0;
         dialog_open(app.buffer,&dlg_days);
         msg=MSG_R_STAY;
         break;
      }
      case MSG_D_QUIT:
      {
         save();
         msg=MSG_R_QUIT;
         break;
      }
      case MSG_KEYPRESS_ASCII:
      {
         if(!isdigit(message.argv[0])) msg=MSG_NOP; /* only digits allowed */
         break;
      }
      case MSG_DIALOG_ACCEPTED:
      {
         int temp=atoi(app.buffer);
         if((temp>0) && (temp)<21)
         {
            app.days=temp;
            app.passages=temp*2;
            save();
            send_message(MSG_REQUEST_QUIT);
         }
         else
         {
            if (app.days>20)
               send_message(MSG_DISPLAY_MESSAGE|MSG_2P,DMI_INFORMATION,&str_msg_cannot_ret);
            else
               send_message(MSG_DISPLAY_MESSAGE|MSG_2P,DMI_INFORMATION,&str_msg_wrong_number);
            dialog_open(app.buffer,&dlg_days);
         }
         msg=MSG_NOP;
         break;
      }
   }
   return msg;
}

/*** passages ***/
ULONG dispatcher_passages(ULONG msg)
{
   switch(msg)
   {
      case MSG_DISPLAY_MENU_VALUE:
      {
         if(app.state!=DEFAULT_STATE)
         {
            init();
         }
         ftoa(app.buffer, app.passages);
         own_print_menu_value((const nokstr_t*)app.buffer);
         break;
      }
      case MSG_D_INIT:
      {
         if(app.state!=DEFAULT_STATE)
         {
            init();
         }
         app.buffer[0]=0;
         dialog_open(app.buffer,&dlg_passages);
         msg=MSG_R_STAY;
         break;
      }
      case MSG_D_QUIT:
      {
         save();
         msg=MSG_R_QUIT;
         break;
      }
      case MSG_KEYPRESS_ASCII:
      {
         if(!isdigit(message.argv[0])) msg=MSG_NOP; /* only digits allowed */
         break;
      }
      case MSG_DIALOG_ACCEPTED:
      {
         int temp=atoi(app.buffer);
         if(temp>0)
         {
            app.passages=temp;
            save();
            send_message(MSG_REQUEST_QUIT);
         }
         else
         {
            send_message(MSG_DISPLAY_MESSAGE|MSG_2P,DMI_INFORMATION,&str_msg_wrong_number);
            dialog_open(app.buffer,&dlg_passages);
         }
         msg=MSG_NOP;
         break;
      }
   }
   return msg;
}

/*** daysinmonth ***/
ULONG dispatcher_daysinmonth(ULONG msg)
{
   switch(msg)
   {
      case MSG_DISPLAY_MENU_VALUE:
      {
         if(app.state!=DEFAULT_STATE)
         {
            init();
         }
         ftoa(app.buffer, app.daysinmonth);
         own_print_menu_value((const nokstr_t*)app.buffer);
         break;
      }
      case MSG_D_INIT:
      {
         if(app.state!=DEFAULT_STATE)
         {
            init();
         }
         app.buffer[0]=0;
         dialog_open(app.buffer,&dlg_daysinmonth);
         msg=MSG_R_STAY;
         break;
      }
      case MSG_D_QUIT:
      {
         save();
         msg=MSG_R_QUIT;
         break;
      }
      case MSG_KEYPRESS_ASCII:
      {
         if(!isdigit(message.argv[0])) msg=MSG_NOP; /* only digits allowed */
         break;
      }
      case MSG_DIALOG_ACCEPTED:
      {
         int temp=atoi(app.buffer);
         if((temp>27) && (temp)<32)
         {
            app.daysinmonth=temp;
            save();
            send_message(MSG_REQUEST_QUIT);
         }
         else
         {
            send_message(MSG_DISPLAY_MESSAGE|MSG_2P,DMI_INFORMATION,&str_msg_wrong_number);
            dialog_open(app.buffer,&dlg_daysinmonth);
         }
         msg=MSG_NOP;
         break;
      }
   }
   return msg;
}

/*** monthcost ***/
ULONG dispatcher_monthcost(ULONG msg)
{
   switch(msg)
   {
      case MSG_DISPLAY_MENU_VALUE:
      {
         if(app.state!=DEFAULT_STATE)
         {
            init();
         }
         ftoa(app.buffer, app.monthcost);
         own_print_menu_value((const nokstr_t*)app.buffer);
         break;
      }
      case MSG_D_INIT:
      {
         if(app.state!=DEFAULT_STATE)
         {
            init();
         }
         app.buffer[0]=0;
         dialog_open(app.buffer,&dlg_monthcost);
         msg=MSG_R_STAY;
         break;
      }
      case MSG_D_QUIT:
      {
         save();
         msg=MSG_R_QUIT;
         break;
      }
      case MSG_KEYPRESS_ASCII:
      {
         if(!isdigit(message.argv[0]) && (message.argv[0]!='*') && (message.argv[0]!='#')) msg=MSG_NOP; /* only digits allowed */
         if((message.argv[0]=='*') || (message.argv[0]=='#'))
            message.argv[0]='.';
         break;
      }
      case MSG_DIALOG_ACCEPTED:
      {
         double temp=atof(app.buffer);
         if(temp>0)
         {
            app.monthcost=temp;
            save();
            send_message(MSG_REQUEST_QUIT);
         }
         else
         {
            send_message(MSG_DISPLAY_MESSAGE|MSG_2P,DMI_INFORMATION,&str_msg_wrong_number);
            dialog_open(app.buffer,&dlg_monthcost);
         }
         msg=MSG_NOP;
         break;
      }
   }
   return msg;
}

/*** oncecost ***/
ULONG dispatcher_oncecost(ULONG msg)
{
   switch(msg)
   {
      case MSG_DISPLAY_MENU_VALUE:
      {
         if(app.state!=DEFAULT_STATE)
         {
            init();
         }
         ftoa(app.buffer, app.oncecost);
         own_print_menu_value((const nokstr_t*)app.buffer);
         break;
      }
      case MSG_D_INIT:
      {
         if(app.state!=DEFAULT_STATE)
         {
            init();
         }
         app.buffer[0]=0;
         dialog_open(app.buffer,&dlg_oncecost);
         msg=MSG_R_STAY;
         break;
      }
      case MSG_D_QUIT:
      {
         save();
         msg=MSG_R_QUIT;
         break;
      }
      case MSG_KEYPRESS_ASCII:
      {
         if(!isdigit(message.argv[0]) && (message.argv[0]!='*') && (message.argv[0]!='#')) msg=MSG_NOP; /* only digits allowed */
         if((message.argv[0]=='*') || (message.argv[0]=='#'))
            message.argv[0]='.';
         break;
      }
      case MSG_DIALOG_ACCEPTED:
      {
         double temp=atof(app.buffer);
         if(temp>0)
         {
            app.oncecost=temp;
            save();
            send_message(MSG_REQUEST_QUIT);
         }
         else
         {
            send_message(MSG_DISPLAY_MESSAGE|MSG_2P,DMI_INFORMATION,&str_msg_wrong_number);
            dialog_open(app.buffer,&dlg_oncecost);
         }
         msg=MSG_NOP;
         break;
      }
   }
   return msg;
}

/*** calculate ***/
ULONG dispatcher_calculate(ULONG msg)
{
   switch(msg)
   {
      case MSG_D_INIT:
      {
         if(app.state!=DEFAULT_STATE)
         {
            init();
         }
         double on_once=app.passages*app.oncecost;
         double on_month=app.monthcost-(0.9*app.monthcost*app.daysinmonth*app.days)/app.daysinmonth;
         if (on_once>on_month)
            send_message(MSG_DISPLAY_MESSAGE|MSG_2P,DMI_INFORMATION,&str_msg_monthly);
         else
            send_message(MSG_DISPLAY_MESSAGE|MSG_2P,DMI_INFORMATION,&str_msg_daily);
         msg=MSG_NOP;
         break;
      }
      case MSG_D_QUIT:
      {
         save();
         msg=MSG_R_QUIT;
         break;
      }
   }
   return msg;
}

